"""CSV parsing and writing."""

import csv
import sys
from _typeshed import SupportsWrite
from collections.abc import Iterable
from typing import Any, Final, Literal, TypeAlias
from typing_extensions import Self, disjoint_base

__version__: Final[str]

QUOTE_ALL: Final = 1
QUOTE_MINIMAL: Final = 0
QUOTE_NONE: Final = 3
QUOTE_NONNUMERIC: Final = 2
if sys.version_info >= (3, 12):
    QUOTE_STRINGS: Final = 4
    QUOTE_NOTNULL: Final = 5

if sys.version_info >= (3, 12):
    _QuotingType: TypeAlias = Literal[0, 1, 2, 3, 4, 5]
else:
    _QuotingType: TypeAlias = Literal[0, 1, 2, 3]

class Error(Exception): ...

_DialectLike: TypeAlias = str | Dialect | csv.Dialect | type[Dialect | csv.Dialect]

@disjoint_base
class Dialect:
    """CSV dialect

    The Dialect type records CSV parsing and generation options.
    """

    delimiter: str
    quotechar: str | None
    escapechar: str | None
    doublequote: bool
    skipinitialspace: bool
    lineterminator: str
    quoting: _QuotingType
    strict: bool
    def __new__(
        cls,
        dialect: _DialectLike | None = None,
        delimiter: str = ",",
        doublequote: bool = True,
        escapechar: str | None = None,
        lineterminator: str = "\r\n",
        quotechar: str | None = '"',
        quoting: _QuotingType = 0,
        skipinitialspace: bool = False,
        strict: bool = False,
    ) -> Self: ...

# This class calls itself _csv.reader.
@disjoint_base
class Reader:
    """CSV reader

    Reader objects are responsible for reading and parsing tabular data
    in CSV format.
    """

    @property
    def dialect(self) -> Dialect: ...
    line_num: int
    def __iter__(self) -> Self:
        """Implement iter(self)."""

    def __next__(self) -> list[str]:
        """Implement next(self)."""

# This class calls itself _csv.writer.
@disjoint_base
class Writer:
    """CSV writer

    Writer objects are responsible for generating tabular data
    in CSV format from sequence input.
    """

    @property
    def dialect(self) -> Dialect: ...
    if sys.version_info >= (3, 13):
        def writerow(self, row: Iterable[Any], /) -> Any:
            """Construct and write a CSV record from an iterable of fields.

            Non-string elements will be converted to string.
            """

        def writerows(self, rows: Iterable[Iterable[Any]], /) -> None:
            """Construct and write a series of iterables to a csv file.

            Non-string elements will be converted to string.
            """

    else:
        def writerow(self, row: Iterable[Any]) -> Any:
            """writerow(iterable)

            Construct and write a CSV record from an iterable of fields.  Non-string
            elements will be converted to string.
            """

        def writerows(self, rows: Iterable[Iterable[Any]]) -> None:
            """writerows(iterable of iterables)

            Construct and write a series of iterables to a csv file.  Non-string
            elements will be converted to string.
            """

def writer(
    fileobj: SupportsWrite[str],
    /,
    dialect: _DialectLike = "excel",
    *,
    delimiter: str = ",",
    quotechar: str | None = '"',
    escapechar: str | None = None,
    doublequote: bool = True,
    skipinitialspace: bool = False,
    lineterminator: str = "\r\n",
    quoting: _QuotingType = 0,
    strict: bool = False,
) -> Writer:
    """Return a writer object that will write user data on the given file object.

    The "fileobj" argument can be any object that supports the file API.
    The optional "dialect" argument defines a CSV dialect.  The function
    also accepts optional keyword arguments which override settings
    provided by the dialect.
    """

def reader(
    iterable: Iterable[str],
    /,
    dialect: _DialectLike = "excel",
    *,
    delimiter: str = ",",
    quotechar: str | None = '"',
    escapechar: str | None = None,
    doublequote: bool = True,
    skipinitialspace: bool = False,
    lineterminator: str = "\r\n",
    quoting: _QuotingType = 0,
    strict: bool = False,
) -> Reader:
    """Return a reader object that will process lines from the given iterable.

    The "iterable" argument can be any object that returns a line
    of input for each iteration, such as a file object or a list.  The
    optional "dialect" argument defines a CSV dialect.  The function
    also accepts optional keyword arguments which override settings
    provided by the dialect.

    The returned object is an iterator.  Each iteration returns a row
    of the CSV file (which can span multiple input lines).
    """

def register_dialect(
    name: str,
    /,
    dialect: type[Dialect | csv.Dialect] | str = "excel",
    *,
    delimiter: str = ",",
    quotechar: str | None = '"',
    escapechar: str | None = None,
    doublequote: bool = True,
    skipinitialspace: bool = False,
    lineterminator: str = "\r\n",
    quoting: _QuotingType = 0,
    strict: bool = False,
) -> None:
    """Create a mapping from a string name to a CVS dialect.

    The optional "dialect" argument specifies the base dialect instance
    or the name of the registered dialect.  The function also accepts
    optional keyword arguments which override settings provided by the
    dialect.
    """

def unregister_dialect(name: str) -> None:
    """Delete the name/dialect mapping associated with a string name."""

def get_dialect(name: str) -> Dialect:
    """Return the dialect instance associated with name."""

def list_dialects() -> list[str]:
    """Return a list of all known dialect names."""

def field_size_limit(new_limit: int = ...) -> int:
    """Sets an upper limit on parsed fields.

    Returns old limit. If limit is not given, no new limit is set and
    the old limit is returned
    """



"""A fast, lightweight IPv4/IPv6 manipulation library in Python.

This library is used to create/poke/manipulate IPv4 and IPv6 addresses
and networks.

"""

import sys
from collections.abc import Iterable, Iterator
from typing import Any, Final, Generic, Literal, TypeAlias, TypeVar, overload
from typing_extensions import Self

# Undocumented length constants
IPV4LENGTH: Final = 32
IPV6LENGTH: Final = 128

_A = TypeVar("_A", IPv4Address, IPv6Address)
_N = TypeVar("_N", IPv4Network, IPv6Network)

_RawIPAddress: TypeAlias = int | str | bytes | IPv4Address | IPv6Address
_RawNetworkPart: TypeAlias = IPv4Network | IPv6Network | IPv4Interface | IPv6Interface

def ip_address(address: _RawIPAddress) -> IPv4Address | IPv6Address:
    """Take an IP string/int and return an object of the correct type.

    Args:
        address: A string or integer, the IP address.  Either IPv4 or
          IPv6 addresses may be supplied; integers less than 2**32 will
          be considered to be IPv4 by default.

    Returns:
        An IPv4Address or IPv6Address object.

    Raises:
        ValueError: if the *address* passed isn't either a v4 or a v6
          address

    """

def ip_network(
    address: _RawIPAddress | _RawNetworkPart | tuple[_RawIPAddress] | tuple[_RawIPAddress, int], strict: bool = True
) -> IPv4Network | IPv6Network:
    """Take an IP string/int and return an object of the correct type.

    Args:
        address: A string or integer, the IP network.  Either IPv4 or
          IPv6 networks may be supplied; integers less than 2**32 will
          be considered to be IPv4 by default.

    Returns:
        An IPv4Network or IPv6Network object.

    Raises:
        ValueError: if the string passed isn't either a v4 or a v6
          address. Or if the network has host bits set.

    """

def ip_interface(
    address: _RawIPAddress | _RawNetworkPart | tuple[_RawIPAddress] | tuple[_RawIPAddress, int],
) -> IPv4Interface | IPv6Interface:
    """Take an IP string/int and return an object of the correct type.

    Args:
        address: A string or integer, the IP address.  Either IPv4 or
          IPv6 addresses may be supplied; integers less than 2**32 will
          be considered to be IPv4 by default.

    Returns:
        An IPv4Interface or IPv6Interface object.

    Raises:
        ValueError: if the string passed isn't either a v4 or a v6
          address.

    Notes:
        The IPv?Interface classes describe an Address on a particular
        Network, so they're basically a combination of both the Address
        and Network classes.

    """

class _IPAddressBase:
    """The mother class."""

    __slots__ = ()
    @property
    def compressed(self) -> str:
        """Return the shorthand version of the IP address as a string."""

    @property
    def exploded(self) -> str:
        """Return the longhand version of the IP address as a string."""

    @property
    def reverse_pointer(self) -> str:
        """The name of the reverse DNS pointer for the IP address, e.g.:
        >>> ipaddress.ip_address("127.0.0.1").reverse_pointer
        '1.0.0.127.in-addr.arpa'
        >>> ipaddress.ip_address("2001:db8::1").reverse_pointer
        '1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.8.b.d.0.1.0.0.2.ip6.arpa'

        """

    if sys.version_info < (3, 14):
        @property
        def version(self) -> int: ...

class _BaseAddress(_IPAddressBase):
    """A generic IP object.

    This IP class contains the version independent methods which are
    used by single IP addresses.
    """

    __slots__ = ()
    def __add__(self, other: int) -> Self: ...
    def __hash__(self) -> int: ...
    def __int__(self) -> int: ...
    def __sub__(self, other: int) -> Self: ...
    def __format__(self, fmt: str) -> str:
        """Returns an IP address as a formatted string.

        Supported presentation types are:
        's': returns the IP address as a string (default)
        'b': converts to binary and returns a zero-padded string
        'X' or 'x': converts to upper- or lower-case hex and returns a zero-padded string
        'n': the same as 'b' for IPv4 and 'x' for IPv6

        For binary and hex presentation types, the alternate form specifier
        '#' and the grouping option '_' are supported.
        """

    def __eq__(self, other: object) -> bool: ...
    def __lt__(self, other: Self) -> bool: ...
    if sys.version_info >= (3, 11):
        def __ge__(self, other: Self) -> bool:
            """Return a >= b.  Computed by @total_ordering from (not a < b)."""

        def __gt__(self, other: Self) -> bool:
            """Return a > b.  Computed by @total_ordering from (not a < b) and (a != b)."""

        def __le__(self, other: Self) -> bool:
            """Return a <= b.  Computed by @total_ordering from (a < b) or (a == b)."""

    else:
        def __ge__(self, other: Self, NotImplemented: Any = ...) -> bool:
            """Return a >= b.  Computed by @total_ordering from (not a < b)."""

        def __gt__(self, other: Self, NotImplemented: Any = ...) -> bool:
            """Return a > b.  Computed by @total_ordering from (not a < b) and (a != b)."""

        def __le__(self, other: Self, NotImplemented: Any = ...) -> bool:
            """Return a <= b.  Computed by @total_ordering from (a < b) or (a == b)."""

class _BaseNetwork(_IPAddressBase, Generic[_A]):
    """A generic IP network object.

    This IP class contains the version independent methods which are
    used by networks.
    """

    network_address: _A
    netmask: _A
    def __contains__(self, other: Any) -> bool: ...
    def __getitem__(self, n: int) -> _A: ...
    def __iter__(self) -> Iterator[_A]: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: Self) -> bool: ...
    if sys.version_info >= (3, 11):
        def __ge__(self, other: Self) -> bool:
            """Return a >= b.  Computed by @total_ordering from (not a < b)."""

        def __gt__(self, other: Self) -> bool:
            """Return a > b.  Computed by @total_ordering from (not a < b) and (a != b)."""

        def __le__(self, other: Self) -> bool:
            """Return a <= b.  Computed by @total_ordering from (a < b) or (a == b)."""

    else:
        def __ge__(self, other: Self, NotImplemented: Any = ...) -> bool:
            """Return a >= b.  Computed by @total_ordering from (not a < b)."""

        def __gt__(self, other: Self, NotImplemented: Any = ...) -> bool:
            """Return a > b.  Computed by @total_ordering from (not a < b) and (a != b)."""

        def __le__(self, other: Self, NotImplemented: Any = ...) -> bool:
            """Return a <= b.  Computed by @total_ordering from (a < b) or (a == b)."""

    def address_exclude(self, other: Self) -> Iterator[Self]:
        """Remove an address from a larger block.

        For example:

            addr1 = ip_network('192.0.2.0/28')
            addr2 = ip_network('192.0.2.1/32')
            list(addr1.address_exclude(addr2)) =
                [IPv4Network('192.0.2.0/32'), IPv4Network('192.0.2.2/31'),
                 IPv4Network('192.0.2.4/30'), IPv4Network('192.0.2.8/29')]

        or IPv6:

            addr1 = ip_network('2001:db8::1/32')
            addr2 = ip_network('2001:db8::1/128')
            list(addr1.address_exclude(addr2)) =
                [ip_network('2001:db8::1/128'),
                 ip_network('2001:db8::2/127'),
                 ip_network('2001:db8::4/126'),
                 ip_network('2001:db8::8/125'),
                 ...
                 ip_network('2001:db8:8000::/33')]

        Args:
            other: An IPv4Network or IPv6Network object of the same type.

        Returns:
            An iterator of the IPv(4|6)Network objects which is self
            minus other.

        Raises:
            TypeError: If self and other are of differing address
              versions, or if other is not a network object.
            ValueError: If other is not completely contained by self.

        """

    @property
    def broadcast_address(self) -> _A: ...
    def compare_networks(self, other: Self) -> int:
        """Compare two IP objects.

        This is only concerned about the comparison of the integer
        representation of the network addresses.  This means that the
        host bits aren't considered at all in this method.  If you want
        to compare host bits, you can easily enough do a
        'HostA._ip < HostB._ip'

        Args:
            other: An IP object.

        Returns:
            If the IP versions of self and other are the same, returns:

            -1 if self < other:
              eg: IPv4Network('192.0.2.0/25') < IPv4Network('192.0.2.128/25')
              IPv6Network('2001:db8::1000/124') <
                  IPv6Network('2001:db8::2000/124')
            0 if self == other
              eg: IPv4Network('192.0.2.0/24') == IPv4Network('192.0.2.0/24')
              IPv6Network('2001:db8::1000/124') ==
                  IPv6Network('2001:db8::1000/124')
            1 if self > other
              eg: IPv4Network('192.0.2.128/25') > IPv4Network('192.0.2.0/25')
                  IPv6Network('2001:db8::2000/124') >
                      IPv6Network('2001:db8::1000/124')

          Raises:
              TypeError if the IP versions are different.

        """

    def hosts(self) -> Iterator[_A]:
        """Generate Iterator over usable hosts in a network.

        This is like __iter__ except it doesn't return the network
        or broadcast addresses.

        """

    @property
    def is_global(self) -> bool:
        """Test if this address is allocated for public networks.

        Returns:
            A boolean, True if the address is not reserved per
            iana-ipv4-special-registry or iana-ipv6-special-registry.

        """

    @property
    def is_link_local(self) -> bool:
        """Test if the address is reserved for link-local.

        Returns:
            A boolean, True if the address is reserved per RFC 4291.

        """

    @property
    def is_loopback(self) -> bool:
        """Test if the address is a loopback address.

        Returns:
            A boolean, True if the address is a loopback address as defined in
            RFC 2373 2.5.3.

        """

    @property
    def is_multicast(self) -> bool:
        """Test if the address is reserved for multicast use.

        Returns:
            A boolean, True if the address is a multicast address.
            See RFC 2373 2.7 for details.

        """

    @property
    def is_private(self) -> bool:
        """Test if this network belongs to a private range.

        Returns:
            A boolean, True if the network is reserved per
            iana-ipv4-special-registry or iana-ipv6-special-registry.

        """

    @property
    def is_reserved(self) -> bool:
        """Test if the address is otherwise IETF reserved.

        Returns:
            A boolean, True if the address is within one of the
            reserved IPv6 Network ranges.

        """

    @property
    def is_unspecified(self) -> bool:
        """Test if the address is unspecified.

        Returns:
            A boolean, True if this is the unspecified address as defined in
            RFC 2373 2.5.2.

        """

    @property
    def num_addresses(self) -> int:
        """Number of hosts in the current subnet."""

    def overlaps(self, other: _BaseNetwork[IPv4Address] | _BaseNetwork[IPv6Address]) -> bool:
        """Tell if self is partly contained in other."""

    @property
    def prefixlen(self) -> int: ...
    def subnet_of(self, other: Self) -> bool:
        """Return True if this network is a subnet of other."""

    def supernet_of(self, other: Self) -> bool:
        """Return True if this network is a supernet of other."""

    def subnets(self, prefixlen_diff: int = 1, new_prefix: int | None = None) -> Iterator[Self]:
        """The subnets which join to make the current subnet.

        In the case that self contains only one IP
        (self._prefixlen == 32 for IPv4 or self._prefixlen == 128
        for IPv6), yield an iterator with just ourself.

        Args:
            prefixlen_diff: An integer, the amount the prefix length
              should be increased by. This should not be set if
              new_prefix is also set.
            new_prefix: The desired new prefix length. This must be a
              larger number (smaller prefix) than the existing prefix.
              This should not be set if prefixlen_diff is also set.

        Returns:
            An iterator of IPv(4|6) objects.

        Raises:
            ValueError: The prefixlen_diff is too small or too large.
                OR
            prefixlen_diff and new_prefix are both set or new_prefix
              is a smaller number than the current prefix (smaller
              number means a larger network)

        """

    def supernet(self, prefixlen_diff: int = 1, new_prefix: int | None = None) -> Self:
        """The supernet containing the current network.

        Args:
            prefixlen_diff: An integer, the amount the prefix length of
              the network should be decreased by.  For example, given a
              /24 network and a prefixlen_diff of 3, a supernet with a
              /21 netmask is returned.

        Returns:
            An IPv4 network object.

        Raises:
            ValueError: If self.prefixlen - prefixlen_diff < 0. I.e., you have
              a negative prefix length.
                OR
            If prefixlen_diff and new_prefix are both set or new_prefix is a
              larger number than the current prefix (larger number means a
              smaller network)

        """

    @property
    def with_hostmask(self) -> str: ...
    @property
    def with_netmask(self) -> str: ...
    @property
    def with_prefixlen(self) -> str: ...
    @property
    def hostmask(self) -> _A: ...

class _BaseV4:
    """Base IPv4 object.

    The following methods are used by IPv4 objects in both single IP
    addresses and networks.

    """

    __slots__ = ()
    if sys.version_info >= (3, 14):
        version: Final = 4
        max_prefixlen: Final = 32
    else:
        @property
        def version(self) -> Literal[4]: ...
        @property
        def max_prefixlen(self) -> Literal[32]: ...

class IPv4Address(_BaseV4, _BaseAddress):
    """Represent and manipulate single IPv4 Addresses."""

    __slots__ = ("_ip", "__weakref__")
    def __init__(self, address: object) -> None:
        """
        Args:
            address: A string or integer representing the IP

              Additionally, an integer can be passed, so
              IPv4Address('192.0.2.1') == IPv4Address(3221225985).
              or, more generally
              IPv4Address(int(IPv4Address('192.0.2.1'))) ==
                IPv4Address('192.0.2.1')

        Raises:
            AddressValueError: If ipaddress isn't a valid IPv4 address.

        """

    @property
    def is_global(self) -> bool:
        """``True`` if the address is defined as globally reachable by
        iana-ipv4-special-registry_ (for IPv4) or iana-ipv6-special-registry_
        (for IPv6) with the following exception:

        For IPv4-mapped IPv6-addresses the ``is_private`` value is determined by the
        semantics of the underlying IPv4 addresses and the following condition holds
        (see :attr:`IPv6Address.ipv4_mapped`)::

            address.is_global == address.ipv4_mapped.is_global

        ``is_global`` has value opposite to :attr:`is_private`, except for the ``100.64.0.0/10``
        IPv4 range where they are both ``False``.
        """

    @property
    def is_link_local(self) -> bool:
        """Test if the address is reserved for link-local.

        Returns:
            A boolean, True if the address is link-local per RFC 3927.

        """

    @property
    def is_loopback(self) -> bool:
        """Test if the address is a loopback address.

        Returns:
            A boolean, True if the address is a loopback per RFC 3330.

        """

    @property
    def is_multicast(self) -> bool:
        """Test if the address is reserved for multicast use.

        Returns:
            A boolean, True if the address is multicast.
            See RFC 3171 for details.

        """

    @property
    def is_private(self) -> bool:
        """``True`` if the address is defined as not globally reachable by
        iana-ipv4-special-registry_ (for IPv4) or iana-ipv6-special-registry_
        (for IPv6) with the following exceptions:

        * ``is_private`` is ``False`` for ``100.64.0.0/10``
        * For IPv4-mapped IPv6-addresses the ``is_private`` value is determined by the
            semantics of the underlying IPv4 addresses and the following condition holds
            (see :attr:`IPv6Address.ipv4_mapped`)::

                address.is_private == address.ipv4_mapped.is_private

        ``is_private`` has value opposite to :attr:`is_global`, except for the ``100.64.0.0/10``
        IPv4 range where they are both ``False``.
        """

    @property
    def is_reserved(self) -> bool:
        """Test if the address is otherwise IETF reserved.

        Returns:
            A boolean, True if the address is within the
            reserved IPv4 Network range.

        """

    @property
    def is_unspecified(self) -> bool:
        """Test if the address is unspecified.

        Returns:
            A boolean, True if this is the unspecified address as defined in
            RFC 5735 3.

        """

    @property
    def packed(self) -> bytes:
        """The binary representation of this address."""

    if sys.version_info >= (3, 13):
        @property
        def ipv6_mapped(self) -> IPv6Address:
            """Return the IPv4-mapped IPv6 address.

            Returns:
                The IPv4-mapped IPv6 address per RFC 4291.

            """

class IPv4Network(_BaseV4, _BaseNetwork[IPv4Address]):
    """This class represents and manipulates 32-bit IPv4 network + addresses..

    Attributes: [examples for IPv4Network('192.0.2.0/27')]
        .network_address: IPv4Address('192.0.2.0')
        .hostmask: IPv4Address('0.0.0.31')
        .broadcast_address: IPv4Address('192.0.2.32')
        .netmask: IPv4Address('255.255.255.224')
        .prefixlen: 27

    """

    def __init__(self, address: object, strict: bool = True) -> None:
        """Instantiate a new IPv4 network object.

        Args:
            address: A string or integer representing the IP [& network].
              '192.0.2.0/24'
              '192.0.2.0/255.255.255.0'
              '192.0.2.0/0.0.0.255'
              are all functionally the same in IPv4. Similarly,
              '192.0.2.1'
              '192.0.2.1/255.255.255.255'
              '192.0.2.1/32'
              are also functionally equivalent. That is to say, failing to
              provide a subnetmask will create an object with a mask of /32.

              If the mask (portion after the / in the argument) is given in
              dotted quad form, it is treated as a netmask if it starts with a
              non-zero field (e.g. /255.0.0.0 == /8) and as a hostmask if it
              starts with a zero field (e.g. 0.255.255.255 == /8), with the
              single exception of an all-zero mask which is treated as a
              netmask == /0. If no mask is given, a default of /32 is used.

              Additionally, an integer can be passed, so
              IPv4Network('192.0.2.1') == IPv4Network(3221225985)
              or, more generally
              IPv4Interface(int(IPv4Interface('192.0.2.1'))) ==
                IPv4Interface('192.0.2.1')

        Raises:
            AddressValueError: If ipaddress isn't a valid IPv4 address.
            NetmaskValueError: If the netmask isn't valid for
              an IPv4 address.
            ValueError: If strict is True and a network address is not
              supplied.
        """

class IPv4Interface(IPv4Address):
    netmask: IPv4Address
    network: IPv4Network
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def hostmask(self) -> IPv4Address: ...
    @property
    def ip(self) -> IPv4Address: ...
    @property
    def with_hostmask(self) -> str: ...
    @property
    def with_netmask(self) -> str: ...
    @property
    def with_prefixlen(self) -> str: ...

class _BaseV6:
    """Base IPv6 object.

    The following methods are used by IPv6 objects in both single IP
    addresses and networks.

    """

    __slots__ = ()
    if sys.version_info >= (3, 14):
        version: Final = 6
        max_prefixlen: Final = 128
    else:
        @property
        def version(self) -> Literal[6]: ...
        @property
        def max_prefixlen(self) -> Literal[128]: ...

class IPv6Address(_BaseV6, _BaseAddress):
    """Represent and manipulate single IPv6 Addresses."""

    __slots__ = ("_ip", "_scope_id", "__weakref__")
    def __init__(self, address: object) -> None:
        """Instantiate a new IPv6 address object.

        Args:
            address: A string or integer representing the IP

              Additionally, an integer can be passed, so
              IPv6Address('2001:db8::') ==
                IPv6Address(42540766411282592856903984951653826560)
              or, more generally
              IPv6Address(int(IPv6Address('2001:db8::'))) ==
                IPv6Address('2001:db8::')

        Raises:
            AddressValueError: If address isn't a valid IPv6 address.

        """

    @property
    def is_global(self) -> bool:
        """``True`` if the address is defined as globally reachable by
        iana-ipv4-special-registry_ (for IPv4) or iana-ipv6-special-registry_
        (for IPv6) with the following exception:

        For IPv4-mapped IPv6-addresses the ``is_private`` value is determined by the
        semantics of the underlying IPv4 addresses and the following condition holds
        (see :attr:`IPv6Address.ipv4_mapped`)::

            address.is_global == address.ipv4_mapped.is_global

        ``is_global`` has value opposite to :attr:`is_private`, except for the ``100.64.0.0/10``
        IPv4 range where they are both ``False``.
        """

    @property
    def is_link_local(self) -> bool:
        """Test if the address is reserved for link-local.

        Returns:
            A boolean, True if the address is reserved per RFC 4291.

        """

    @property
    def is_loopback(self) -> bool:
        """Test if the address is a loopback address.

        Returns:
            A boolean, True if the address is a loopback address as defined in
            RFC 2373 2.5.3.

        """

    @property
    def is_multicast(self) -> bool:
        """Test if the address is reserved for multicast use.

        Returns:
            A boolean, True if the address is a multicast address.
            See RFC 2373 2.7 for details.

        """

    @property
    def is_private(self) -> bool:
        """``True`` if the address is defined as not globally reachable by
        iana-ipv4-special-registry_ (for IPv4) or iana-ipv6-special-registry_
        (for IPv6) with the following exceptions:

        * ``is_private`` is ``False`` for ``100.64.0.0/10``
        * For IPv4-mapped IPv6-addresses the ``is_private`` value is determined by the
            semantics of the underlying IPv4 addresses and the following condition holds
            (see :attr:`IPv6Address.ipv4_mapped`)::

                address.is_private == address.ipv4_mapped.is_private

        ``is_private`` has value opposite to :attr:`is_global`, except for the ``100.64.0.0/10``
        IPv4 range where they are both ``False``.
        """

    @property
    def is_reserved(self) -> bool:
        """Test if the address is otherwise IETF reserved.

        Returns:
            A boolean, True if the address is within one of the
            reserved IPv6 Network ranges.

        """

    @property
    def is_unspecified(self) -> bool:
        """Test if the address is unspecified.

        Returns:
            A boolean, True if this is the unspecified address as defined in
            RFC 2373 2.5.2.

        """

    @property
    def packed(self) -> bytes:
        """The binary representation of this address."""

    @property
    def ipv4_mapped(self) -> IPv4Address | None:
        """Return the IPv4 mapped address.

        Returns:
            If the IPv6 address is a v4 mapped address, return the
            IPv4 mapped address. Return None otherwise.

        """

    @property
    def is_site_local(self) -> bool:
        """Test if the address is reserved for site-local.

        Note that the site-local address space has been deprecated by RFC 3879.
        Use is_private to test if this address is in the space of unique local
        addresses as defined by RFC 4193.

        Returns:
            A boolean, True if the address is reserved per RFC 3513 2.5.6.

        """

    @property
    def sixtofour(self) -> IPv4Address | None:
        """Return the IPv4 6to4 embedded address.

        Returns:
            The IPv4 6to4-embedded address if present or None if the
            address doesn't appear to contain a 6to4 embedded address.

        """

    @property
    def teredo(self) -> tuple[IPv4Address, IPv4Address] | None:
        """Tuple of embedded teredo IPs.

        Returns:
            Tuple of the (server, client) IPs or None if the address
            doesn't appear to be a teredo address (doesn't start with
            2001::/32)

        """

    @property
    def scope_id(self) -> str | None:
        """Identifier of a particular zone of the address's scope.

        See RFC 4007 for details.

        Returns:
            A string identifying the zone of the address if specified, else None.

        """

    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...

class IPv6Network(_BaseV6, _BaseNetwork[IPv6Address]):
    """This class represents and manipulates 128-bit IPv6 networks.

    Attributes: [examples for IPv6('2001:db8::1000/124')]
        .network_address: IPv6Address('2001:db8::1000')
        .hostmask: IPv6Address('::f')
        .broadcast_address: IPv6Address('2001:db8::100f')
        .netmask: IPv6Address('ffff:ffff:ffff:ffff:ffff:ffff:ffff:fff0')
        .prefixlen: 124

    """

    def __init__(self, address: object, strict: bool = True) -> None:
        """Instantiate a new IPv6 Network object.

        Args:
            address: A string or integer representing the IPv6 network or the
              IP and prefix/netmask.
              '2001:db8::/128'
              '2001:db8:0000:0000:0000:0000:0000:0000/128'
              '2001:db8::'
              are all functionally the same in IPv6.  That is to say,
              failing to provide a subnetmask will create an object with
              a mask of /128.

              Additionally, an integer can be passed, so
              IPv6Network('2001:db8::') ==
                IPv6Network(42540766411282592856903984951653826560)
              or, more generally
              IPv6Network(int(IPv6Network('2001:db8::'))) ==
                IPv6Network('2001:db8::')

            strict: A boolean. If true, ensure that we have been passed
              A true network address, eg, 2001:db8::1000/124 and not an
              IP address on a network, eg, 2001:db8::1/124.

        Raises:
            AddressValueError: If address isn't a valid IPv6 address.
            NetmaskValueError: If the netmask isn't valid for
              an IPv6 address.
            ValueError: If strict was True and a network address was not
              supplied.
        """

    @property
    def is_site_local(self) -> bool:
        """Test if the address is reserved for site-local.

        Note that the site-local address space has been deprecated by RFC 3879.
        Use is_private to test if this address is in the space of unique local
        addresses as defined by RFC 4193.

        Returns:
            A boolean, True if the address is reserved per RFC 3513 2.5.6.

        """

class IPv6Interface(IPv6Address):
    netmask: IPv6Address
    network: IPv6Network
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def hostmask(self) -> IPv6Address: ...
    @property
    def ip(self) -> IPv6Address: ...
    @property
    def with_hostmask(self) -> str: ...
    @property
    def with_netmask(self) -> str: ...
    @property
    def with_prefixlen(self) -> str: ...

def v4_int_to_packed(address: int) -> bytes:
    """Represent an address as 4 packed bytes in network (big-endian) order.

    Args:
        address: An integer representation of an IPv4 IP address.

    Returns:
        The integer address packed as 4 bytes in network (big-endian) order.

    Raises:
        ValueError: If the integer is negative or too large to be an
          IPv4 IP address.

    """

def v6_int_to_packed(address: int) -> bytes:
    """Represent an address as 16 packed bytes in network (big-endian) order.

    Args:
        address: An integer representation of an IPv6 IP address.

    Returns:
        The integer address packed as 16 bytes in network (big-endian) order.

    """

# Third overload is technically incorrect, but convenient when first and last are return values of ip_address()
@overload
def summarize_address_range(first: IPv4Address, last: IPv4Address) -> Iterator[IPv4Network]:
    """Summarize a network range given the first and last IP addresses.

    Example:
        >>> list(summarize_address_range(IPv4Address('192.0.2.0'),
        ...                              IPv4Address('192.0.2.130')))
        ...                                #doctest: +NORMALIZE_WHITESPACE
        [IPv4Network('192.0.2.0/25'), IPv4Network('192.0.2.128/31'),
         IPv4Network('192.0.2.130/32')]

    Args:
        first: the first IPv4Address or IPv6Address in the range.
        last: the last IPv4Address or IPv6Address in the range.

    Returns:
        An iterator of the summarized IPv(4|6) network objects.

    Raise:
        TypeError:
            If the first and last objects are not IP addresses.
            If the first and last objects are not the same version.
        ValueError:
            If the last object is not greater than the first.
            If the version of the first address is not 4 or 6.

    """
@overload
def summarize_address_range(first: IPv6Address, last: IPv6Address) -> Iterator[IPv6Network]: ...
@overload
def summarize_address_range(
    first: IPv4Address | IPv6Address, last: IPv4Address | IPv6Address
) -> Iterator[IPv4Network] | Iterator[IPv6Network]: ...

def collapse_addresses(addresses: Iterable[_N]) -> Iterator[_N]:
    """Collapse a list of IP objects.

    Example:
        collapse_addresses([IPv4Network('192.0.2.0/25'),
                            IPv4Network('192.0.2.128/25')]) ->
                           [IPv4Network('192.0.2.0/24')]

    Args:
        addresses: An iterable of IPv4Network or IPv6Network objects.

    Returns:
        An iterator of the collapsed IPv(4|6)Network objects.

    Raises:
        TypeError: If passed a list of mixed version objects.

    """

@overload
def get_mixed_type_key(obj: _A) -> tuple[int, _A]:
    """Return a key suitable for sorting between networks and addresses.

    Address and Network objects are not sortable by default; they're
    fundamentally different so the expression

        IPv4Address('192.0.2.0') <= IPv4Network('192.0.2.0/24')

    doesn't make any sense.  There are some times however, where you may wish
    to have ipaddress sort these for you anyway. If you need to do this, you
    can use this function as the key= argument to sorted().

    Args:
      obj: either a Network or Address object.
    Returns:
      appropriate key.

    """
@overload
def get_mixed_type_key(obj: IPv4Network) -> tuple[int, IPv4Address, IPv4Address]: ...
@overload
def get_mixed_type_key(obj: IPv6Network) -> tuple[int, IPv6Address, IPv6Address]: ...

class AddressValueError(ValueError):
    """A Value Error related to the address."""

class NetmaskValueError(ValueError):
    """A Value Error related to the netmask."""



"""Context Variables"""

import sys
from collections.abc import Callable, Iterator, Mapping
from types import GenericAlias, TracebackType
from typing import Any, ClassVar, Generic, ParamSpec, TypeVar, final, overload
from typing_extensions import Self

_T = TypeVar("_T")
_D = TypeVar("_D")
_P = ParamSpec("_P")

@final
class ContextVar(Generic[_T]):
    @overload
    def __new__(cls, name: str) -> Self: ...
    @overload
    def __new__(cls, name: str, *, default: _T) -> Self: ...

    def __hash__(self) -> int: ...
    @property
    def name(self) -> str: ...

    @overload
    def get(self) -> _T:
        """Return a value for the context variable for the current context.

        If there is no value for the variable in the current context, the
        method will:
         * return the value of the default argument of the method, if
           provided; or
         * return the default value for the context variable, if it was
           created with one; or
         * raise a LookupError.
        """
    @overload
    def get(self, default: _T, /) -> _T: ...
    @overload
    def get(self, default: _D, /) -> _D | _T: ...

    def set(self, value: _T, /) -> Token[_T]:
        """Call to set a new value for the context variable in the current context.

        The required value argument is the new value for the context
        variable.

        Returns a Token object that can be used to restore the variable to
        its previous value via the `ContextVar.reset()` method.
        """

    def reset(self, token: Token[_T], /) -> None:
        """Reset the context variable.

        The variable is reset to the value it had before the
        `ContextVar.set()` that created the token was used.
        """

    def __class_getitem__(cls, item: Any, /) -> GenericAlias:
        """ContextVars are generic over the type of their contained values"""

@final
class Token(Generic[_T]):
    @property
    def var(self) -> ContextVar[_T]: ...
    @property
    def old_value(self) -> Any: ...  # returns either _T or MISSING, but that's hard to express
    MISSING: ClassVar[object]
    __hash__: ClassVar[None]  # type: ignore[assignment]
    def __class_getitem__(cls, item: Any, /) -> GenericAlias:
        """Tokens are generic over the same type as the ContextVar which created them."""

    if sys.version_info >= (3, 14):
        def __enter__(self) -> Self:
            """Enter into Token context manager."""

        def __exit__(
            self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None, /
        ) -> None:
            """Exit from Token context manager, restore the linked ContextVar."""

def copy_context() -> Context: ...

# It doesn't make sense to make this generic, because for most Contexts each ContextVar will have
# a different value.
@final
class Context(Mapping[ContextVar[Any], Any]):
    def __init__(self) -> None: ...

    @overload
    def get(self, key: ContextVar[_T], default: None = None, /) -> _T | None:
        """Return the value for `key` if `key` has the value in the context object.

        If `key` does not exist, return `default`.  If `default` is not
        given, return None.
        """
    @overload
    def get(self, key: ContextVar[_T], default: _T, /) -> _T: ...
    @overload
    def get(self, key: ContextVar[_T], default: _D, /) -> _T | _D: ...

    def run(self, callable: Callable[_P, _T], *args: _P.args, **kwargs: _P.kwargs) -> _T: ...
    def copy(self) -> Context:
        """Return a shallow copy of the context object."""

    __hash__: ClassVar[None]  # type: ignore[assignment]
    def __getitem__(self, key: ContextVar[_T], /) -> _T:
        """Return self[key]."""

    def __iter__(self) -> Iterator[ContextVar[Any]]:
        """Implement iter(self)."""

    def __len__(self) -> int:
        """Return len(self)."""

    def __eq__(self, value: object, /) -> bool: ...



# On invalid indentation, recover as if the indentation wasn't there
if True:
    pass
        a + b

    pass

a = 10

# Multiple nested unexpected indents.
if True:
    before_nested
        first_nested
            second_nested
    after_nested

outside_nested

# A valid compound statement inside recovered indentation.
if True:
    before_compound
        if condition:
            nested_compound
        recovered_compound
    after_compound

outside_compound

# Multiple independent unexpected-indent regions in the same body.
if True:
    before_regions
        first_region
    middle_region
        second_region
    after_region

outside_regions

# An independent syntax error inside recovered indentation stays visible.
if True:
    before_error
        broken(,)
    after_error

outside_error

# Outstanding unexpected indents are flushed at EOF.
if True:
    before_eof
        final_eof



# Test cases for fmt: skip on compound statements that fit on one line

# Basic single-line compound statements
def simple_func(): return "hello"  # fmt: skip
if True: print("condition met")  # fmt: skip
for i in range(5): print(i)  # fmt: skip
while x < 10: x += 1  # fmt: skip

# With expressions that would normally trigger formatting
def long_params(a, b, c, d, e, f, g): return a + b + c + d + e + f + g  # fmt: skip
if some_very_long_condition_that_might_wrap: do_something_else_that_is_long()  # fmt: skip

# Nested compound statements (outer should be preserved)
if True:
    for i in range(10): print(i)  # fmt: skip

# Multiple statements in body (should not apply - multiline)
if True:
    x = 1
    y   =   2  # fmt: skip

# With decorators - decorated function on one line
@overload
def decorated_func(x: int) -> str: return str(x)  # fmt: skip

@property
def prop_method(self): return self._value  # fmt: skip

# Class definitions on one line
class SimpleClass: pass  # fmt: skip
class GenericClass(Generic[T]): pass  # fmt: skip

# Try/except blocks
try: risky_operation()  # fmt: skip
except ValueError: handle_error()  # fmt: skip
except: handle_any_error()  # fmt: skip
else: success_case()  # fmt: skip
finally: cleanup()  # fmt: skip

# Match statements (Python 3.10+)
match value:
    case 1: print("one")  # fmt: skip
    case _: print("other")  # fmt: skip

# With statements
with open("file.txt") as f: content = f.read()  # fmt: skip
with context_manager() as cm: result = cm.process()  # fmt: skip

# Async variants
async def async_func(): return await some_call()  # fmt: skip
async for item in async_iterator(): await process(item)  # fmt: skip
async with async_context() as ctx: await ctx.work()  # fmt: skip

# Complex expressions that would normally format
def complex_expr(): return [x for x in range(100) if x % 2 == 0 and x > 50]  # fmt: skip
if condition_a and condition_b or (condition_c and not condition_d): execute_complex_logic()  # fmt: skip

# Edge case: comment positioning
def func_with_comment():  # some comment
    return "value"  # fmt: skip

# Edge case: multiple fmt: skip (only last one should matter)
def multiple_skip(): return "test"  # fmt: skip  # fmt: skip

# Should NOT be affected (already multiline)
def multiline_func():
    return "this should format normally"

if long_condition_that_spans \
   and continues_on_next_line:
    print("multiline condition")

# Mix of skipped and non-skipped
for i in range(10): print(f"item {i}")  # fmt: skip
for j in range(5):
    print(f"formatted item {j}")

# With trailing comma that would normally be removed
def trailing_comma_func(a, b, c,): return a + b + c  # fmt: skip

# Dictionary/list comprehensions
def dict_comp(): return {k: v for k, v in items.items() if v is not None}  # fmt: skip
def list_comp(): return [x * 2 for x in numbers if x > threshold_value]  # fmt: skip

# Lambda in one-liner
def with_lambda(): return lambda x, y, z: x + y + z if all([x, y, z]) else None  # fmt: skip

# String formatting that would normally be reformatted
def format_string(): return f"Hello {name}, you have {count} items in your cart totaling ${total:.2f}"  # fmt: skip

# loop else clauses
for i in range(2): print(i) # fmt: skip
else: print("this") # fmt: skip


while foo(): print(i) # fmt: skip
else: print("this") # fmt: skip

# again but only the first skip
for i in range(2): print(i) # fmt: skip
else: print("this")


while foo(): print(i) # fmt: skip
else: print("this")

# again but only the second skip
for i in range(2): print(i)
else: print("this") # fmt: skip


while foo(): print(i)
else: print("this") # fmt: skip

# multiple statements in body
if True: print("this"); print("that") # fmt: skip

# Examples with more comments

try: risky_operation()  # fmt: skip
# leading 1
except ValueError: handle_error()  # fmt: skip
# leading 2
except: handle_any_error()  # fmt: skip
# leading 3
else: success_case()  # fmt: skip
# leading 4
finally: cleanup()  # fmt: skip
# trailing

# multi-line before colon (should remain as is)
if (
	long_condition
):  a     +     b # fmt: skip

# over-indented comment example
# See https://github.com/astral-sh/ruff/pull/20633#issuecomment-3453288910
# and https://github.com/astral-sh/ruff/pull/21185

for x in it: foo()
    # comment
else: bar() # fmt: skip


if this(
    'is a long',
    # commented
    'condition'
): with_a_skip # fmt: skip



import sys
from collections.abc import Callable
from mmap import mmap
from multiprocessing import popen_forkserver, popen_spawn_posix, resource_sharer
from typing import Protocol, TypeAlias, type_check_only

__all__ = ["BufferWrapper"]

class Arena:
    """
    A shared memory area backed by a temporary file (POSIX).
    """

    size: int
    buffer: mmap
    if sys.platform == "win32":
        name: str
        def __init__(self, size: int) -> None: ...
    else:
        fd: int
        def __init__(self, size: int, fd: int = -1) -> None: ...

_Block: TypeAlias = tuple[Arena, int, int]

if sys.platform != "win32":
    @type_check_only
    class _SupportsDetach(Protocol):
        def detach(self) -> int: ...

    def reduce_arena(
        a: Arena,
    ) -> tuple[
        Callable[[int, _SupportsDetach], Arena],
        tuple[int, popen_forkserver._DupFd | popen_spawn_posix._DupFd | resource_sharer.DupFd],
    ]: ...
    def rebuild_arena(size: int, dupfd: _SupportsDetach) -> Arena: ...

class Heap:
    def __init__(self, size: int = ...) -> None: ...
    def free(self, block: _Block) -> None: ...
    def malloc(self, size: int) -> _Block: ...

class BufferWrapper:
    def __init__(self, size: int) -> None: ...
    def create_memoryview(self) -> memoryview: ...



"""Operator interface.

This module exports a set of functions implemented in C corresponding
to the intrinsic operators of Python.  For example, operator.add(x, y)
is equivalent to the expression x+y.  The function names are those
used for special methods; variants without leading and trailing
'__' are also provided for convenience.
"""

import sys
from _typeshed import (
    SupportsAdd,
    SupportsGetItem,
    SupportsMod,
    SupportsMul,
    SupportsRAdd,
    SupportsRMod,
    SupportsRMul,
    SupportsRSub,
    SupportsSub,
)
from collections.abc import Callable, Container, Iterable, MutableMapping, MutableSequence, Sequence
from operator import attrgetter as attrgetter, itemgetter as itemgetter, methodcaller as methodcaller
from typing import Any, AnyStr, ParamSpec, Protocol, SupportsAbs, SupportsIndex, TypeAlias, TypeVar, overload, type_check_only
from typing_extensions import TypeIs

_R = TypeVar("_R")
_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)
_T_contra = TypeVar("_T_contra", contravariant=True)
_K = TypeVar("_K")
_V = TypeVar("_V")
_P = ParamSpec("_P")

# The following protocols return "Any" instead of bool, since the comparison
# operators can be overloaded to return an arbitrary object. For example,
# the numpy.array comparison dunders return another numpy.array.

@type_check_only
class _SupportsDunderLT(Protocol):
    def __lt__(self, other: Any, /) -> Any: ...

@type_check_only
class _SupportsDunderGT(Protocol):
    def __gt__(self, other: Any, /) -> Any: ...

@type_check_only
class _SupportsDunderLE(Protocol):
    def __le__(self, other: Any, /) -> Any: ...

@type_check_only
class _SupportsDunderGE(Protocol):
    def __ge__(self, other: Any, /) -> Any: ...

_SupportsComparison: TypeAlias = _SupportsDunderLE | _SupportsDunderGE | _SupportsDunderGT | _SupportsDunderLT

@type_check_only
class _SupportsInversion(Protocol[_T_co]):
    def __invert__(self) -> _T_co: ...

@type_check_only
class _SupportsNeg(Protocol[_T_co]):
    def __neg__(self) -> _T_co: ...

@type_check_only
class _SupportsPos(Protocol[_T_co]):
    def __pos__(self) -> _T_co: ...

# All four comparison functions must have the same signature, or we get false-positive errors
def lt(a: _SupportsComparison, b: _SupportsComparison, /) -> Any:
    """Same as a < b."""

def le(a: _SupportsComparison, b: _SupportsComparison, /) -> Any:
    """Same as a <= b."""

def eq(a: object, b: object, /) -> Any:
    """Same as a == b."""

def ne(a: object, b: object, /) -> Any:
    """Same as a != b."""

def ge(a: _SupportsComparison, b: _SupportsComparison, /) -> Any:
    """Same as a >= b."""

def gt(a: _SupportsComparison, b: _SupportsComparison, /) -> Any:
    """Same as a > b."""

def not_(a: object, /) -> bool:
    """Same as not a."""

def truth(a: object, /) -> bool:
    """Return True if a is true, False otherwise."""

def is_(a: object, b: object, /) -> bool:
    """Same as a is b."""

def is_not(a: object, b: object, /) -> bool:
    """Same as a is not b."""

def abs(a: SupportsAbs[_T], /) -> _T:
    """Same as abs(a)."""

@overload
def add(a: SupportsAdd[_T_contra, _T_co], b: _T_contra, /) -> _T_co:
    """Same as a + b."""
@overload
def add(a: _T_contra, b: SupportsRAdd[_T_contra, _T_co], /) -> _T_co: ...

def and_(a, b, /):
    """Same as a & b."""

def floordiv(a, b, /):
    """Same as a // b."""

def index(a: SupportsIndex, /) -> int:
    """Same as a.__index__()"""

def inv(a: _SupportsInversion[_T_co], /) -> _T_co:
    """Same as ~a."""

def invert(a: _SupportsInversion[_T_co], /) -> _T_co:
    """Same as ~a."""

def lshift(a, b, /):
    """Same as a << b."""

@overload
def mod(a: SupportsMod[_T_contra, _T_co], b: _T_contra, /) -> _T_co:
    """Same as a % b."""
@overload
def mod(a: _T_contra, b: SupportsRMod[_T_contra, _T_co], /) -> _T_co: ...

@overload
def mul(a: SupportsMul[_T_contra, _T_co], b: _T_contra, /) -> _T_co:
    """Same as a * b."""
@overload
def mul(a: _T_contra, b: SupportsRMul[_T_contra, _T_co], /) -> _T_co: ...

def matmul(a, b, /):
    """Same as a @ b."""

def neg(a: _SupportsNeg[_T_co], /) -> _T_co:
    """Same as -a."""

def or_(a, b, /):
    """Same as a | b."""

def pos(a: _SupportsPos[_T_co], /) -> _T_co:
    """Same as +a."""

def pow(a, b, /):
    """Same as a ** b."""

def rshift(a, b, /):
    """Same as a >> b."""

@overload
def sub(a: SupportsSub[_T_contra, _T_co], b: _T_contra, /) -> _T_co:
    """Same as a - b."""
@overload
def sub(a: _T_contra, b: SupportsRSub[_T_contra, _T_co], /) -> _T_co: ...

def truediv(a, b, /):
    """Same as a / b."""

def xor(a, b, /):
    """Same as a ^ b."""

def concat(a: Sequence[_T], b: Sequence[_T], /) -> Sequence[_T]:
    """Same as a + b, for a and b sequences."""

def contains(a: Container[object], b: object, /) -> bool:
    """Same as b in a (note reversed operands)."""

def countOf(a: Iterable[object], b: object, /) -> int:
    """Return the number of items in a which are, or which equal, b."""

@overload
def delitem(a: MutableSequence[Any], b: int, /) -> None:
    """Same as del a[b]."""
@overload
def delitem(a: MutableSequence[Any], b: slice[int | None], /) -> None: ...
@overload
def delitem(a: MutableMapping[_K, Any], b: _K, /) -> None: ...

@overload
def getitem(a: Sequence[_T], b: slice[int | None], /) -> Sequence[_T]:
    """Same as a[b]."""
@overload
def getitem(a: SupportsGetItem[_K, _V], b: _K, /) -> _V: ...

def indexOf(a: Iterable[_T], b: _T, /) -> int:
    """Return the first index of b in a."""

@overload
def setitem(a: MutableSequence[_T], b: int, c: _T, /) -> None:
    """Same as a[b] = c."""
@overload
def setitem(a: MutableSequence[_T], b: slice[int | None], c: Sequence[_T], /) -> None: ...
@overload
def setitem(a: MutableMapping[_K, _V], b: _K, c: _V, /) -> None: ...

def length_hint(obj: object, default: int = 0, /) -> int:
    """Return an estimate of the number of items in obj.

    This is useful for presizing containers when building from an iterable.

    If the object supports len(), the result will be exact.
    Otherwise, it may over- or under-estimate by an arbitrary amount.
    The result will be an integer >= 0.
    """

def iadd(a, b, /):
    """Same as a += b."""

def iand(a, b, /):
    """Same as a &= b."""

def iconcat(a, b, /):
    """Same as a += b, for a and b sequences."""

def ifloordiv(a, b, /):
    """Same as a //= b."""

def ilshift(a, b, /):
    """Same as a <<= b."""

def imod(a, b, /):
    """Same as a %= b."""

def imul(a, b, /):
    """Same as a *= b."""

def imatmul(a, b, /):
    """Same as a @= b."""

def ior(a, b, /):
    """Same as a |= b."""

def ipow(a, b, /):
    """Same as a **= b."""

def irshift(a, b, /):
    """Same as a >>= b."""

def isub(a, b, /):
    """Same as a -= b."""

def itruediv(a, b, /):
    """Same as a /= b."""

def ixor(a, b, /):
    """Same as a ^= b."""

if sys.version_info >= (3, 11):
    def call(obj: Callable[_P, _R], /, *args: _P.args, **kwargs: _P.kwargs) -> _R:
        """Same as obj(*args, **kwargs)."""

def _compare_digest(a: AnyStr, b: AnyStr, /) -> bool:
    """Return 'a == b'.

    This function uses an approach designed to prevent
    timing analysis, making it appropriate for cryptography.

    a and b must both be of the same type: either str (ASCII only),
    or any bytes-like object.

    Note: If a and b are of different lengths, or if an error occurs,
    a timing attack could theoretically reveal information about the
    types and lengths of a and b--but not their values.
    """

if sys.version_info >= (3, 14):
    def is_none(a: object, /) -> TypeIs[None]:
        """Same as a is None."""

    def is_not_none(a: _T | None, /) -> TypeIs[_T]:
        """Same as a is not None."""
